@@ -1,4 +1,3 @@
-
 ################################################################################
 #################                    Encoding                  #################
 ################################################################################
@@ -20,11 +19,11 @@ import json
 from .globals_ import OUTPUT_DIR, OUTPUT_FOLDER, DATA_DIR
 from .aaindex import  AAIndex
 from .model import Model
-from .proDSP import ProDSP
+from .pyDSP import PyDSP
 from .evaluate import Evaluate
 from .pySAR import PySAR
 from .utils import *
-from .descriptors import Descriptors
+from .descriptors_ import Descriptors
 from .plots import plot_reg
 
 class Encoding(PySAR):
@@ -52,11 +51,11 @@ class Encoding(PySAR):
 
     Attributes
     ----------
-    **Refer to pySAR module doctring for description of parameters
+        :config_file (str): path to configuration file.
 
     Methods
     -------
-    aai_encoding(self, use_dsp=True, aai_list=None, spectrum='power',window='hamming',filter_="")
+    aai_encoding(self, aai_list=None, cutoff_index=1):
         encoding protein sequences using indices from the AAI.
     descriptor_encoding(desc_combo=1):
         encoding protein sequences using protein descriptors from descrtipors module.
@@ -65,67 +64,53 @@ class Encoding(PySAR):
         with the protein descriptors from the descriptors module.
     """
 
-    def __init__(
-        self, dataset="",seq_col="sequence", activity="", \
-            algorithm="", parameters={}, test_split=0.2,descriptors_csv="descriptors.csv"
-            ):
+    def __init__(self, config_file=""):
 
-        super().__init__(
-            dataset=dataset,seq_col=seq_col, \
-                activity=activity, algorithm=algorithm, parameters=parameters,
-                test_split=test_split, descriptors_csv=descriptors_csv
-                )
+        self.config_file = conf= config_file
+
+        super().__init__(self.config_file)
 
         #create output directory to store all of the program's outputs
         create_output_dir()
 
-    def aai_encoding(
-        self, use_dsp=True, aai_list=None, spectrum='power',window='hamming',
-        filter_=None, cutoff_index=1
-        ):
+    def aai_encoding(self, aai_list=None, cutoff_index=1, sort_by='R2'):
         """
         Encoding all protein sequences using each of the available indices in the
         AAI. The protein spectra of the AAI indices will be generated if use_dsp is true,
-        dictated by the instance attributes: spectrum, window and filter. If not true then
-        the encoced sequences from the AAI will be used. Each encoding will be
+        dictated by the instance attributes: spectrum, window, convolution and filter. 
+        If not true then the encoced sequences from the AAI will be used. Each encoding will be
         used as the feature data to build the predictive regression models. To date,
         there are 566 indices in the AAI, therefore 566 total models can be built using
         this encoding strategy. The metrics evaluated from the model for each AAI
-        encoding combination will be collated into a dataframe, saved and returned.
+        encoding combination will be collated into a dataframe, saved and returned, with the 
+        results sorted by R2 by default, this can be changed using the sort_by parameter. 
 
         Parameters
         ----------
-        use_dsp : bool (default = True)
-            if true then pass AAI indices encoding through a DSP pipeline to
-            create the protein spectra, dictated by instance attributes: spectrum,
-            window and filter. If false then AAI indices encoding used as feature data.
-        aai_list : list (default = None)
+        :aai_list : list (default = None)
             list of aai indices to use for encoding the predictive models, by default
             all AAI indices will be used.
-        spectrum : str (default = 'power')
-            protein spectra to generate from the FFT of the protein sequences.
-        window : str (default = 'hamming')
-            window function to apply to output of FFT on the protein sequences.
-        filter : str (default = None)
-            filter function to apply to output of FFT on the protein sequences.
-        cutoff_index : int (default = 1)
-            **refer to aai_encoding(...) docstring.
+        :cutoff_index : int (default = 1)
+            set the proportion of iterations of AAI indices to actually encode -> mainly used for testing.
+        :sort_by : str (default = R2)
+            sort output dataframe by specified column/metric value, results sorted by R2 score by default.
 
         Returns
         -------
-        aaindex_metrics_df : pd.DataFrame
+        :aaindex_metrics_df : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
             encoded using indices in the AAI for the AAI encoding strategy.
         """
         #setting DSP params to None if not using them
-        if !(use_dsp):
-            spectrum = None
-            window = None
-            filter_ = None
+        if not (self.use_dsp):
+            self.spectrum = None
+            self.window = None
+            self.filter = None
+            self.convolution = None
 
         #initialise dataframe to store all output results of AAI encoding
         aaindex_metrics_df = pd.DataFrame(columns=['Index','Category','R2', 'RMSE',
-            'MSE', 'RPD', 'MAE', 'Explained Var'])
+            'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
         aa_list = []
@@ -141,51 +126,53 @@ class Encoding(PySAR):
 
         #if no indices passed into aai_list then use all indices by default
         if aai_list == None or aai_list == [] or aai_list == "":
-            all_indices = self.aaindex.get_record_codes()
-        elif isinstance(aai_list,str):   #if single descriptor input, cast to list
+            all_indices = self.aaindex.record_codes()
+        elif isinstance(aai_list, str):   #if single descriptor input, cast to list
             all_indices = [aai_list]
         else:
             all_indices = aai_list
 
         print('\n#######################################################################################\n')
-        print('Encoding using {} AAI combination(s) with the parameters:\n\n# Spectrum -> {}\n# Window Function -> {} \
-            \n# Filter -> {}\n# Algorithm -> {}\n# Parameters -> {}\n# Test Split -> {}\n'.format(len(all_indices), spectrum,\
-            window, filter_, repr(self.model), self.model.model.get_params(), self.test_split))
-        print('#######################################################################################\n')
+        print('Encoding using {} AAI combination(s) with the parameters:\n\n# Algorithm -> {}\n# Parameters -> {} \
+            \n# Test Split -> {}'.format(len(all_indices), repr(self.model), self.model.model.get_params(), self.test_split))
+        if (self.use_dsp):
+            print('# Using DSP -> {}\n# Spectrum -> {}\n# Window Function -> {}\n# Filter -> {}\n# Convolution -> {}'.format(
+                self.use_dsp, self.spectrum, self.window, self.filter, self.convolution
+            ))
+        print('\n#######################################################################################\n')
 
         #set the proportion of iterations to actually complete -> mainly used for testing
         cutoff_index = int(len(all_indices) * cutoff_index)
 
         '''
         1.) Get AAI index encoding of protein sequences, if using DSP (use_dsp = True),
-        create instance of proDSP class and generate protein spectra from the AAI
-        indices, according to instance parameters: spectrum, window and filter.
+        create instance of pyDSP class and generate protein spectra from the AAI
+        indices, according to instance parameters: spectrum, window, convolution and filter.
         2.) Build model using encoded AAI indices or protein spectra as features.
         3.) Predict and evaluate the model using the test data.
         4.) Append index and calculated metrics to lists.
         5.) Repeat steps 1 - 4 for all indices.
         6.) Output results into a final dataframe, save it and return.
         '''
-        start = time.time() #start counter
+        start = time.time() #start time counter
 
         #using tqdm package to create a progress bar showing encoding progress
         #   file=sys.stdout to stop error where iterations were printing out of order
-        for index in tqdm(all_indices[:cutoff_index],unit=" indices",position=0, desc="AAI Indices",file=sys.stdout):
+        for index in tqdm(all_indices[:cutoff_index],unit=" indices",position=0,desc="AAI Indices",file=sys.stdout):
 
             #get AAI indices encoding for sequences according to index var
-            encoded_seqs = self.get_aai_enoding(index)
+            encoded_seqs = self.get_aai_encoding(index)
 
-            #generate protein spectra from proDSP class if use_dsp is true
-            if use_dsp:
-                proDSP = ProDSP(encoded_seqs, spectrum=spectrum,
-                    window=window, filter_=filter_)
-                proDSP.encode_seqs()
-                X = pd.DataFrame(proDSP.spectrum_encoding)
+            #generate protein spectra from pyDSP class if use_dsp is true
+            if self.use_dsp:
+                pyDSP = PyDSP(self.config_file, protein_seqs=encoded_seqs)
+                pyDSP.encode_seqs()
+                X = pd.DataFrame(pyDSP.spectrum_encoding)
             else:
                 X = pd.DataFrame(encoded_seqs)
 
             #get observed activity values from the dataset
-            Y  = self.get_activity()
+            Y = self.get_activity()
 
             #split feature data and labels into train and test data
             X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X, Y)
@@ -209,7 +196,7 @@ class Encoding(PySAR):
             mae_.append(eval.mae)
             explained_var_.append(eval.explained_var)
 
-        end = time.time()       #stop counter
+        end = time.time()       #stop time counter
         elapsed = end - start
         print('\n\n##############################################################')
         print('Elapsed Time for AAI Encoding: {0:.3f} seconds'.format(elapsed))
@@ -223,47 +210,54 @@ class Encoding(PySAR):
         aaindex_metrics_['MSE'] = mse_
         aaindex_metrics_['RPD'] = rpd_
         aaindex_metrics_['MAE'] = mae_
-        aaindex_metrics_['Explained Var'] = explained_var_
+        aaindex_metrics_['Explained Variance'] = explained_var_
+
+        #sort output dataframe by sort_by parameter, sorted by R2 by default
+        if (sort_by not in aaindex_metrics_df.columns):
+            sort_by = 'R2'
 
         #sort results by R2
-        aaindex_metrics_ = aaindex_metrics_.sort_values(by=['R2'],ascending=False)
+        aaindex_metrics_ = aaindex_metrics_.sort_values(by=[sort_by],ascending=False)
 
         #save results dataframe, saved to OUTPUT_DIR by default.
         save_results(aaindex_metrics_,'aaindex_results')
 
         return aaindex_metrics_
 
-    def descriptor_encoding(self, desc_list=None, desc_combo=1, cutoff_index=1):
+    def descriptor_encoding(self, desc_list=None, desc_combo=1, cutoff_index=1, sort_by='R2'):
         """
         Encoding all protein sequences using the available physiochemical
         and structural descriptors. The sequences can be encoded using combinations
         of 1, 2 or 3 of these descriptors, dictated by the desc_combo input parameter:
-        set this to 1,2 or 3 for what encoding combination to use, default is 1. Each
+        set this to 1, 2 or 3 for what encoding combination to use, default is 1. Each
         descriptor encoding will be used as the feature data to build the predictive
         regression models. With 15 descriptors supported by pySAR this means there
         can be 15, 105 and 455 total predictive models built for 1, 2 or 3 descriptors,
         respecitvely. The metrics evaluated from the model for each Descriptor encoding
-        combination will be collated into a dataframe and saved and returned.
+        combination will be collated into a dataframe and saved and returned, with the 
+        results sorted by the R2 score, this can be changed using the sort_by parameter.
 
         Parameters
         ----------
-        desc_combo : int (default = 1)
+        :desc_combo : int (default = 1)
             combination of descriptors to use.
-        decs_list : list (default = None)
+        :decs_list : list (default = None)
             list of descriptors to use for encoding, by default all available descriptors
             in the descriptors module will be used for the encoding.
-        cutoff_index : int (default = 1)
-            **refer to aai_encoding(...) docstring.
+        :cutoff_index : int (default = 1)
+            set the proportion of iterations of descriptors to actually encode -> mainly used for testing.
+        :sort_by : str (default = R2)
+            sort output dataframe by specified column/metric value, results sorted by R2 score by default.
 
         Returns
         -------
-        desc_metrics_df_ : pd.DataFrame
+        :desc_metrics_df_ : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
             encoded using all descriptors for the descriptors encoding strategy.
         """
         #create dataframe to store output results from models
         desc_metrics_df = pd.DataFrame(columns=['Descriptor','Group','R2', 'RMSE',
-            'MSE', 'RPD', 'MAE', 'Explained Var'])
+            'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
         descriptor = []
@@ -279,14 +273,16 @@ class Encoding(PySAR):
 
         #if no descriptors passed into desc_list then use all descriptors by default,
         #   get list of  all descriptors according to desc_combo value
+        desc = Descriptors(self.config_file)
+        # print('ctd here', desc.ctd)
         if desc_list == None or desc_list == [] or desc_list == "":
-            desc = Descriptors(self.data[self.seq_col], all_desc = True)
+            # desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
         elif isinstance(desc_list,str):     #if single descriptor input, cast to list
-            desc = Descriptors(self.data[self.seq_col])
+            # desc = Descriptors(self.config_file)
             all_descriptors = [desc_list]
         else:
-            desc = Descriptors(self.data[self.seq_col])
+            # desc = Descriptors(self.config_file)
             if desc_combo == 2:
                 all_descriptors = list(itertools.combinations(desc_list, 2))
             elif desc_combo == 3:
@@ -311,7 +307,7 @@ class Encoding(PySAR):
         3.) Predict and evaluate the model using the test data.
         4.) Append descriptor(s) and calculated metrics to lists.
         5.) Repeat steps 1 - 4 for all descriptors.
-        6.) Output results into a final dataframe, save it and return.
+        6.) Output results into a final dataframe, save it and return, sorting by sort_by parameter.
         '''
         for descr in tqdm(all_descriptors[:cutoff_index],unit=" descriptor",position=0,desc="Descriptors",file=sys.stdout):
 
@@ -322,25 +318,24 @@ class Encoding(PySAR):
             #if using 2 or 3 descriptors, append each descriptor & its category to list
             if desc_combo == 2 or desc_combo == 3:
                 for de in descr:
-
                     descriptor_list.append(getattr(desc, de))
                     descriptor_group_.append(desc.descriptor_groups[de])
-
                 desc_ = pd.concat(descriptor_list,axis=1) #concatenate descriptors
-
             else:
                 # desc_ = getattr(desc, descr)
                 desc_ = desc.get_descriptor_encoding(descr)
                 descriptor_group_.append(desc.descriptor_groups[descr])
                 # descriptor_group_.append(desc.descriptor_groups[descr])
 
-            X = desc_
+            X = desc_   #set training data to desc_ dataframe
 
             #get protein activity values
             Y  = self.get_activity()
-
+            print(descr)
+            print(X)
+            # print(Y.shape)
             '''
-            if using the PlsRegression algorithm and there is only 1 feature (1-dimension)
+            If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
             in the feature data X then create a new PLSReg model with the n_components
             parameter set to 1 instead of the default 2 - this stops the error:
             'ValueError - Invalid Number of Components: 2'
@@ -349,12 +344,11 @@ class Encoding(PySAR):
             '''
             if X.shape[1] == 1 and repr(self.model) == "PLSRegression":
               tmp_model = Model('plsreg',parameters={'n_components':1})
-              X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X, Y, test_size=self.test_split)
+              X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X,Y,test_size=self.test_split)
               model_fit = tmp_model.fit()
               Y_pred = tmp_model.predict()
-
             else:
-              X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X, Y, test_size=self.test_split)
+              X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X,Y,test_size=self.test_split)
               model_fit = self.model.fit()
               Y_pred = self.model.predict()
 
@@ -393,10 +387,14 @@ class Encoding(PySAR):
         desc_metrics_df_['MSE'] = mse_
         desc_metrics_df_['RPD'] = rpd_
         desc_metrics_df_['MAE'] = mae_
-        desc_metrics_df_['Explained Var'] = explained_var_
+        desc_metrics_df_['Explained Variance'] = explained_var_
+
+        #sort output dataframe by sort_by parameter, sorted by R2 by default
+        if (sort_by not in desc_metrics_df_.columns):
+            sort_by = 'R2'
 
         #sort results by R2
-        desc_metrics_df_ = desc_metrics_df_.sort_values(by=['R2'],ascending=False)
+        desc_metrics_df_ = desc_metrics_df_.sort_values(by=[sort_by],ascending=False)
 
         #set save path according to the descriptor combinations type
         if desc_combo == 2:
@@ -411,10 +409,7 @@ class Encoding(PySAR):
 
         return desc_metrics_df_
 
-    def aai_descriptor_encoding(
-        self, aai_list=None, desc_list=None, desc_combo=1, use_dsp=True,
-        spectrum='power', window='hamming', filter_=None, cutoff_index=1
-        ):
+    def aai_descriptor_encoding(self, aai_list=None,desc_list=None,desc_combo=1,cutoff_index=1,sort_by='R2'):
         """
         Encoding all protein sequences using each of the indices in the AAI as well
         as the protein descriptors. The sequences can be encoded using 1 AAI + 1 Descriptor,
@@ -427,42 +422,37 @@ class Encoding(PySAR):
         descriptors so the encoding process will generate 8490, ~59000 and ~257000
         models, when using 1, 2 or 3 descriptors + AAI indices, respectively. The
         metrics evaluated from the model for each AAI + Descriptor encoding combination
-        will be collated into a dataframe and saved and returned.
+        will be collated into a dataframe and saved and returned, sorted by R2 score by default.
 
         Parameters
         ----------
-        aai_list : list (default = None)
+        :aai_list : list (default = None)
             **refer to aai_encoding(...) docstring.
-        desc_list : list (default = None)
+        :desc_list : list (default = None)
             **refer to desc_encoding(...) docstring.
-        desc_combo : int (default = 1)
+        :desc_combo : int (default = 1)
             **refer to desc_encoding(...) docstring.
-        use_dsp : bool (default = True)
+        :cutoff_index : int (default = 1)
             **refer to aai_encoding(...) docstring.
-        spectrum : str (default = 'power')
-            **refer to aai_encoding(...) docstring.
-        window : str (default = 'hamming')
-            **refer to aai_encoding(...) docstring.
-        filter : str (default = None)
-            **refer to aai_encoding(...) docstring.
-        cutoff_index : int (default = 1)
+        :sort_by : str (default = R2)
             **refer to aai_encoding(...) docstring.
 
         Returns
         -------
-        aai_desc_metrics_df_ : pd.DataFrame
+        :aai_desc_metrics_df_ : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
             encoded using AAI indices + descriptors encoding strategy.
         """
         #setting DSP params to None if not using them
-        if !(use_dsp):
-            spectrum = None
-            window = None
-            filter_ = None
+        if not (self.use_dsp):
+            self.spectrum = None
+            self.window = None
+            self.filter_ = None
+            self.convolution = None
 
         #create dataframe to store output results from models
         aai_desc_metrics_df = pd.DataFrame(columns=['Index','Category', 'Descriptor',\
-            'Descriptor Group','R2', 'RMSE', 'MSE', 'RPD', 'MAE', 'Explained Var'])
+            'Descriptor Group','R2', 'RMSE', 'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
         index_ = []
@@ -475,13 +465,14 @@ class Encoding(PySAR):
         rpd_ = []
         mae_ = []
         explained_var_ = []
+
         index_count = 1     #counters to keep track of current index & descriptor
         desc_count = 1
 
         #if no indices passed into aai_list then use all indices by default
         if aai_list == None or aai_list == [] or aai_list == "":
             all_indices = self.aaindex.get_record_codes()
-        elif isinstance(aai_list,str):   #if single descriptor input, cast to list
+        elif isinstance(aai_list, str):   #if single descriptor input, cast to list
             all_indices = [aai_list]
         else:
             all_indices = aai_list
@@ -489,13 +480,16 @@ class Encoding(PySAR):
         #if no descriptors passed into desc_list then use all descriptors by default,
         #   get list of all descriptors according to desc_combo value
         if desc_list == None or desc_list == [] or desc_list == "":
-            desc = Descriptors(self.data[self.seq_col], all_desc = True)
+            # desc = Descriptors(self.data[self.sequence_col], all_desc = True)
+            desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
         elif isinstance(desc_list,str):     #if single descriptor input, cast to list
-            desc = Descriptors(self.data[self.seq_col])
+            # desc = Descriptors(self.data[self.sequence_col])
+            desc = Descriptors(self.config_file)
             all_descriptors = [desc_list]
         else:
-            desc = Descriptors(self.data[self.seq_col])
+            # desc = Descriptors(self.data[self.sequence_col])
+            desc = Descriptors(self.config_file)
             if desc_combo == 2:
                 all_descriptors = list(itertools.combinations(desc_list, 2))
             elif desc_combo == 3:
@@ -505,9 +499,9 @@ class Encoding(PySAR):
 
         print('\n\n##############################################################\n')
         print('Encoding using {} AAI and {} descriptor combination(s) with the parameters:\n\
-            \n# Window -> {}\n# Filter -> {}\n# Spectrum -> {}\n# Algorithm -> {}\n# Parameters -> {}\
+            \n# Window -> {}\n# Filter -> {}\n# Spectrum -> {}\n# Convolution -> {}\n# Algorithm -> {}\n# Parameters -> {}\
             \n# Test Split -> {}\n'.format(len(all_indices), len(all_descriptors),
-                window, filter_, spectrum, repr(self.model), self.parameters, self.test_split))
+                self.window, self.filter_, self.spectrum, self.convolution, repr(self.model), self.parameters, self.test_split))
         print('##################################################################\n')
 
         #set the proportion of iterations to actually complete -> mainly used for testing
@@ -517,29 +511,29 @@ class Encoding(PySAR):
 
         '''
         1.) Get AAI index encoding of protein sequences. If using DSP, create instance
-        of proDSP class and generate protein spectra from the AAI indices, according to
-        instance parameters: spectrum, window and filter.
-        2.) Get all 15 descriptor values and concatenate to AAI encoding features.
+        of pyDSP class and generate protein spectra from the AAI indices, according to
+        instance parameters: spectrum, window, convolution and filter.
+        2.) Get all descriptor values and concatenate to AAI encoding features.
         3.) Build model using concatenated AAI and Descriptor features as the training data.
         4.) Predict and evaluate the model using the test data.
         5.) Append index, descriptor and calculated metrics to lists.
         6.) Repeat steps 1 - 5 for all indices in the AAI.
-        7.) Output results into a final dataframe, save it and return.
+        7.) Output results into a final dataframe, save it and return, sort by sort_by parameter.
         '''
         for index in tqdm(all_indices[:cutoff_index],unit=" indices",desc="AAI Indices"):
 
             #get AAI indices encoding for sequences according to index var
-            encoded_seqs = self.get_aai_enoding(index)
+            encoded_seqs = self.get_aai_encoding(index)
 
-            #generate protein spectra from proDSP class if use_dsp is true
+            #generate protein spectra from pyDSP class if use_dsp is true
             if use_dsp:
-                proDSP = ProDSP(encoded_seqs, spectrum=spectrum,
-                    window=window, filter_=filter_)
-                proDSP.encode_seqs()
-                X_aai = pd.DataFrame(proDSP.spectrum_encoding)
+                pyDSP = PyDSP(self.config_file, protein_seqs=encoded_seqs)
+                pyDSP.encode_seqs()
+                X_aai = pd.DataFrame(pyDSP.spectrum_encoding)
             else:
                 X_aai = pd.DataFrame(encoded_seqs)
 
+            #iterate through all descriptors
             for descr in tqdm(all_descriptors, leave=False,unit=" descriptor",desc="Descriptors"):
 
                 #reset descriptor DF and list
@@ -550,11 +544,9 @@ class Encoding(PySAR):
                 #if using 2 or 3 descriptors, append each descriptor & its category to list
                 if desc_combo == 2 or desc_combo == 3:
                     for de in descr:
-
                         descriptor_list.append(getattr(desc, de)) #get descriptor attribute
                         # descriptor_list.append(desc.get_descriptor_encoding(de))
                         descriptor_group_.append(desc.descriptor_groups[de])
-
                     desc_ = pd.concat(descriptor_list,axis=1) # check if this is needed here
 
                 #if only using 1 descriptor
@@ -563,13 +555,13 @@ class Encoding(PySAR):
                     # desc_ = desc.get_descriptor_encoding(descr)
                     descriptor_group_.append(desc.descriptor_groups[descr])
 
-                X = desc_
+                X = desc_   #set training data to descriptor dataframe
 
                 #get protein activity values
                 Y  = self.get_activity()
 
                 '''
-                if using the PlsRegression algorithm and there is only 1 feature (1-dimension)
+                If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
                 in the feature data X then create a new PLSReg model with the n_components
                 parameter set to 1 instead of the default 2 - this stops the error:
                 'ValueError - Invalid Number of Components: 2.'
@@ -581,7 +573,6 @@ class Encoding(PySAR):
                   X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X, Y)
                   model_fit = tmp_model.fit()
                   Y_pred = tmp_model.predict()
-
                 else:
                   X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X, Y)
                   model_fit = self.model.fit()
@@ -628,10 +619,14 @@ class Encoding(PySAR):
         aai_desc_metrics_df_['MSE'] = mse_
         aai_desc_metrics_df_['RPD'] = rpd_
         aai_desc_metrics_df_['MAE'] = mae_
-        aai_desc_metrics_df_['Explained Var'] = explained_var_
+        aai_desc_metrics_df_['Explained Variance'] = explained_var_
+
+        #sort output dataframe by sort_by parameter, sorted by R2 by default
+        if (sort_by not in aai_desc_metrics_df_.columns):
+            sort_by = 'R2'
 
         #sort results by R2
-        aai_desc_metrics_df_ = aai_desc_metrics_df_.sort_values(by=['R2'],ascending=False)
+        aai_desc_metrics_df_ = aai_desc_metrics_df_.sort_values(by=[sort_by],ascending=False)
 
         #set save path according to the descriptor combinations type
         if desc_combo == 2:
@@ -646,6 +641,8 @@ class Encoding(PySAR):
 
         return aai_desc_metrics_df_
 
+################################################################################
+
     def __str__(self):
         return "Instance of Encoding Class with attribute values: \
                 \nDataset: {}\n, Activity: {}\n,Algorithm: {}\n Parameters: {}\nTest Split: {}\n".format(
