@@ -1,4 +1,3 @@
-
 ################################################################################
 #################                    Evaluate                  #################
 ################################################################################
@@ -12,27 +11,34 @@ class Evaluate():
     """
     An instance of the Evaluate class will calculate various metric values for
     the inputted observed (Y_true) and predicted (Y_pred) arrays, storing the
-    results in the class attributed. Evaluate supports metrics: R2, RMSE, MSE,
+    results in the class attributes. The class supports metrics: R2, RMSE, MSE,
     MAE, RPD, Explained Variance, Max Error and Mean Poisson Deviance.
 
     Attributes
     ----------
-    Y_true : np.ndarray
+    :Y_true : np.ndarray
         array of observed activity values.
-    Y_pred : np.ndarray
-        array of predicted activity values
+    :Y_pred : np.ndarray
+        array of predicted activity values.
 
     Methods
     -------
-    r2_(multioutput='uniform_average')
-    rmse_(multioutput='uniform_average')
-    mse_(multioutput='uniform_average')
-    mae_(multioutput='uniform_average')
-    rpd_()
-    explained_var_(multioutput='uniform_average')
-    max_error_()
-    mean_poisson_deviance_()
-
+    r2_(multioutput='uniform_average'):
+        calculate R2 score.
+    rmse_(multioutput='uniform_average'):
+        calculate RMSE value
+    mse_(multioutput='uniform_average'):
+        calculate MSE value.
+    mae_(multioutput='uniform_average'):
+        calculate MAE value.
+    rpd_():
+        calculate ratio of performance to deviation.
+    explained_var_(multioutput='uniform_average'):
+        calculate explained variance.
+    max_error_():
+        calculate max errror.
+    mean_poisson_deviance_():
+        calculate mean poisson deviance.
     """
     def __init__(self, Y_true, Y_pred):
 
@@ -53,6 +59,8 @@ class Evaluate():
         self.mae = self.mae_()
         self.rpd = self.rpd_()
         self.explained_var = self.explained_var_()
+        self.mean_poisson_deviance = self.mean_poisson_deviance_()
+        self.max_error = self.max_error_()
 
     def r2_(self, multioutput='uniform_average'):
         """
@@ -60,13 +68,13 @@ class Evaluate():
 
         Parameters
         ----------
-        multioutput : str (default = 'uniform_average')
+        :multioutput : str (default = 'uniform_average')
             method that defines aggregating of multiple output scores. Default
-            is reccomended ('uniform_average'), available values -
+            is reccomended ('uniform_average'), available values:
             {‘raw_values’, ‘uniform_average’, ‘variance_weighted’}.
         Returns
         -------
-        r2 : float
+        :r2 : float
             R2 (coefficient of determination) score for observed and predicted values.
         """
         return r2_score(self.Y_true, self.Y_pred, multioutput=multioutput)
@@ -78,13 +86,13 @@ class Evaluate():
 
         Parameters
         ----------
-        multioutput : str (default = 'uniform_average')
+        :multioutput : str (default = 'uniform_average')
             method that defines aggregating of multiple output scores. Default
-            is reccomended ('uniform_average'), available values -
+            is reccomended ('uniform_average'), available values:
             {‘raw_values’, ‘uniform_average’, ‘variance_weighted’}.
         Returns
         -------
-        mse : float
+        :mse : float
             MSE (mean square error) score for observed and predicted values.
         """
         return mean_squared_error(self.Y_true, self.Y_pred, multioutput=multioutput)
@@ -97,29 +105,32 @@ class Evaluate():
 
         Parameters
         ----------
-        multioutput : str (default = 'uniform_average')
+        :multioutput : str (default = 'uniform_average')
             method that defines aggregating of multiple output scores. Default
-            is reccomended ('uniform_average'), available values -
+            is reccomended ('uniform_average'), available values:
             {‘raw_values’, ‘uniform_average’, ‘variance_weighted’}.
         Returns
         -------
-        mse : float
+        :rmse : float
             RMSE score for observed and predicted values.
         """
         return mean_squared_error(self.Y_true, self.Y_pred, squared=False, multioutput=multioutput)
 
     def mae_(self, multioutput='uniform_average'):
         """
-        Calculate the Mean absolute error regression loss.
+        Calculate the Mean Absolute Error (MAE) regression loss for
+        inputted observed and predicted values.
 
         Parameters
         ----------
-        multioutput : str
-            Defines aggregating of multiple output scores. Array-like value
-            defines weights used to average scores.
+        :multioutput : str (default = 'uniform_average')
+            method that defines aggregating of multiple output scores. Default
+            is reccomended ('uniform_average'), available values:
+            {‘raw_values’, ‘uniform_average’, ‘variance_weighted’}.
+
         Returns
         -------
-        mae : float
+        :mae : float
             If multioutput is ‘raw_values’, then MAE is returned for each output
             separately. If multioutput is ‘uniform_average’ or an ndarray of
             weights, then the weighted average of all output errors is returned.
@@ -129,30 +140,32 @@ class Evaluate():
 
     def rpd_(self):
         """
-        Calculates the ratio of performance to deviation (RPD). RPD is the ratio
+        Calculates the Ratio of Performance to Deviation (RPD). RPD is the ratio
         between the standard deviation of a variable and the standard error of
         prediction of that variable by a given model.
 
         Returns
         -------
-        rpd : float
+        :rpd : float
             the RPD score for the model.
         """
         return self.Y_true.std()/np.sqrt(self.mse_())
 
     def explained_var_(self, multioutput='uniform_average'):
         """
-        Calculates the explained variance regression score. Best possible score is 1.0,
+        Calculates the Explained Variance regression score. Best possible score is 1.0,
         lower values are worse.
 
         Parameters
         ----------
-        multioutput : str (default = 'uniform_average')
-            Defines aggregating of multiple output scores. Array-like value
-            defines weights used to average scores.
+        :multioutput : str (default = 'uniform_average')
+            method that defines aggregating of multiple output scores. Default
+            is reccomended ('uniform_average'), available values:
+            {‘raw_values’, ‘uniform_average’, ‘variance_weighted’}.
+
         Returns
         -------
-        explained_var : float
+        :explained_var : float
             The explained variance or ndarray if ‘multioutput’ is ‘raw_values’.
         """
         return explained_variance_score(self.Y_true, self.Y_pred, multioutput=multioutput)
@@ -164,29 +177,31 @@ class Evaluate():
 
         Returns
         -------
-        max_error : float
+        :max_error : float
             A positive floating point value (the best value is 0.0).
         """
         return float(max_error(self.Y_true, self.Y_pred))
 
-    def mean_poisson_deviance(self):
+    def mean_poisson_deviance_(self):
         """
         Calculate Mean Poisson deviance regression loss.
 
         Returns
         -------
-        mean_poisson_deviance : float
+        :mean_poisson_deviance : float
             A non-negative floating point value (the best value is 0.0).
         """
         return mean_poisson_deviance(self.Y_true, self.Y_true)
 
+################################################################################
+
     def __repr__(self):
         return "<Evaluate(Y_true: {} Y_pred: {})>".format(
-            self.Y_true.shape, self.Y_pred.shape
-        )
+            self.Y_true.shape, self.Y_pred.shape)
 
     def __str__(self):
         return "Instance of Evaluate Class with attribute values: \
-                R2: {}, RMSE: {}, MSE: {}, MAE: {}, RPD: {} Explained Variance: {}" \
-                .format(self.r2, self.rmse, self.mse, self.mae, self.rpd,self.explained_var
-        )
+                R2: {}, RMSE: {}, MSE: {}, MAE: {}, RPD: {}, Explained Variance: {}, \
+                    Max Error: {}, Mean Poisson Deviance: {}" \
+                .format(self.r2,self.rmse,self.mse,self.mae,self.rpd,self.explained_var,
+                    self.max_error,self.mean_poisson_deviance)
