@@ -22,9 +22,11 @@ class AAIndexTests(unittest.TestCase):
 
     @unittest.skip("Don't want to overload the FTP server each time tests are run.")
     def test_download(self):
-        """ Test Case to check that the download functionality works for the required AAI1 file.
+        """ 
+        Test Case to check that the download functionality works for the required AAI1 file.
         The file will firstly be removed from the test directory and then redownloaded, and
-        its presence in the directory will pass the test. """
+        its presence in the directory will pass the test. 
+        """
 #1.)
         #if AAI1 present in test dir then remove
         if (os.path.isfile(os.path.join(self.TEST_DIR, 'test_aaindex1'))):
@@ -43,9 +45,10 @@ class AAIndexTests(unittest.TestCase):
 
     @unittest.skip("Don't want to overload the FTP server each time tests are run.")
     def test_url(self):
-        """ Test Case to check that the URL endpoints used for downloading the AAI databases
-        return a 200 status code. """
-
+        """ 
+        Test Case to check that the URL endpoints used for downloading the AAI databases
+        return a 200 status code. 
+        """
         AA_INDEX1_URL_FTP = "https://www.genome.jp/ftp/db/community/aaindex/aaindex1"
         AA_INDEX1_URL_HTTPS = "https://www.genome.jp/ftp/db/community/aaindex/aaindex1"
         AA_INDEX2_URL = "https://www.genome.jp/ftp/db/community/aaindex/aaindex2"
@@ -71,7 +74,9 @@ class AAIndexTests(unittest.TestCase):
         #maybe try requests.mock
 
     def test_get_amino_acids(self):
-        """ Test Case to check that only valid Amino Acids are used within the AAIndex class. """
+        """ 
+        Test Case to check that only valid Amino Acids are used within the AAIndex class. 
+        """
 
         valid_amino_acids = ['-', 'A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K',
             'L', 'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y']
@@ -82,9 +87,11 @@ class AAIndexTests(unittest.TestCase):
                 amino acids: {}.'.format(aa, valid_amino_acids))
 
     def test_num_features(self):
-        """ Test Case to check that the correct number of indices/records are present
+        """ 
+        Test Case to check that the correct number of indices/records are present
         in the AAIndex object. To date, 566 indices are present in the database,
-        features may be added to the AAI in time so the test takes this into account. """
+        features may be added to the AAI in time so the test takes this into account. 
+        """
 #1.)
         self.assertTrue(self.aaindex.get_num_records() >= 566, 'Incorrect number \
             of features found in AAI, wanted 566, got {}.'.format(self.aaindex.get_num_records()))
@@ -94,12 +101,14 @@ class AAIndexTests(unittest.TestCase):
         the number of keys in parsed JSON from the AAI is correct. The test takes
         into account if more features are added to the database in the future. """
 #1.)
-        self.assertTrue(len(self.aaindex.get_record_codes())>=566, 'Incorrect number \
-            of features found in AAI, wanted 566, got {}.'.format(len(self.aaindex.get_record_codes())))
+        self.assertTrue(len(self.aaindex.record_codes())>=566, 'Incorrect number \
+            of features found in AAI, wanted 566, got {}.'.format(len(self.aaindex.record_codes())))
 
     def test_get_record_from_code(self):
-        """ Test Case to check that the correct amino acid values are returned for
-        the feature/index codes using the get_record_from_code function. """
+        """ 
+        Test Case to check that the correct amino acid values are returned for
+        the feature/index codes using the get_record_from_code function. 
+        """
 #1.)
         #initialise test feature codes and their correct amino acid values
         feature1 = 'AURR980103'
@@ -124,7 +133,7 @@ class AAIndexTests(unittest.TestCase):
             'Y': 4.0, 'I': 6.7, 'V': 6.8, '-': 0}
 #1.)
         #get amino acid values for inputted feature/index codes
-        record = self.aaindex.get_record_from_code(feature1)
+        record = self.aaindex[feature1]
 
         self.assertIn('description', list(record.keys()))
         self.assertIn('refs', list(record.keys()))
@@ -137,7 +146,7 @@ class AAIndexTests(unittest.TestCase):
         self.assertEqual(record['notes'],'')
         self.assertTrue(record['refs'].startswith("Aurora, R. and Rose, G."))
 #2.)
-        record = self.aaindex.get_record_from_code(feature2)
+        record = self.aaindex[feature2]
 
         self.assertIn('description', list(record.keys()))
         self.assertIn('refs', list(record.keys()))
@@ -150,7 +159,7 @@ class AAIndexTests(unittest.TestCase):
         self.assertEqual(record['notes'],'')
         self.assertTrue(record['refs'].startswith("Fauchere, J.L., Charton, M"))
 #3.)
-        record = self.aaindex.get_record_from_code(feature3)
+        record = self.aaindex[feature3]
 
         self.assertIn('description', list(record.keys()))
         self.assertIn('refs', list(record.keys()))
@@ -163,7 +172,7 @@ class AAIndexTests(unittest.TestCase):
         self.assertEqual(record['notes'],'')
         self.assertTrue(record['refs'].startswith("Nagano, K."))
 #4.)
-        record = self.aaindex.get_record_from_code(feature4)
+        record = self.aaindex[feature4]
         self.assertIn('description', list(record.keys()))
         self.assertIn('refs', list(record.keys()))
         self.assertIn('notes', list(record.keys()))
@@ -181,33 +190,49 @@ class AAIndexTests(unittest.TestCase):
         feature7 = "blahblahblah"
 
         with self.assertRaises(ValueError):
-            feature_vals = self.aaindex.get_record_from_code(feature5)
+            feature_vals = self.aaindex[feature5]
 
         with self.assertRaises(ValueError):
-            feature_vals = self.aaindex.get_record_from_code(feature6)
+            feature_vals = self.aaindex[feature6]
 
         with self.assertRaises(ValueError):
-            feature_vals = self.aaindex.get_record_from_code(feature7)
+            feature_vals = self.aaindex[feature7]
 
     def test_get_record_from_name(self):
-
-        feature1 = 'YUTK870101'
-        # Unfolding Gibbs energy in water, pH7.0 (Yutani et al., 1987)
-        feature2 = 'ZIMJ680103'
-# Polarity (Zimmerman et al., 1968)
-        feature3 = 'ROSM880103'
-# Loss of Side chain hydropathy by helix formation (Roseman, 1988)
+        """ 
+        Test case for get_record_from_name function that returns an AAI record or its closest
+        match from user-input name/description, also testing that empty list returned if no 
+        found match in database. 
+        """
+        # YUTK870101 
+        feature1 = 'Unfolding Gibbs energy in water, pH7.0 (Yutani et al., 1987)'
+        # ZIMJ680103 
+        feature2 = 'Polarity (Zimmerman et al., 1968)'
+        # ROSM880103 
+        feature3 = 'Loss of Side chain hydropathy by helix formation (Roseman, 1988)'
 
         error_feature1 = 'blahblahblah'
         error_feature2 = 'not a record name'
         error_feature3 = 'also not a record name'
-
+#1.)
+        self.assertEqual(self.aaindex.get_record_from_name(feature1)[0]['description'], feature1)
+        self.assertEqual(self.aaindex.get_record_from_name(feature2)[0]['description'], feature2)
+        self.assertEqual(self.aaindex.get_record_from_name(feature3)[0]['description'], feature3)
+#2.)
+        self.assertEqual(self.aaindex.get_record_from_name(error_feature1), [])
+        self.assertEqual(self.aaindex.get_record_from_name(error_feature2), [])
+        self.assertEqual(self.aaindex.get_record_from_name(error_feature3), [])
+#3.)
+        self.assertTrue(len(self.aaindex.get_record_from_name(feature1, all_matches=1)), 3)
+        self.assertTrue(len(self.aaindex.get_record_from_name(feature2, all_matches=1)), 3)
+        self.assertTrue(len(self.aaindex.get_record_from_name(feature3, all_matches=1)), 3)
 
     def test_record_codes(self):
-        """ Test Case to check that correct feature/record codes are in the parsed JSON
+        """ 
+        Test Case to check that correct feature/record codes are in the parsed JSON
         of all AAI records. Also testing that random erroneous feature codes are
-        not present. """
-
+        not present. 
+        """
         #testing actual index codes
         feature1 = 'VHEG790101'
         feature2 = 'PONP800107'
@@ -215,13 +240,13 @@ class AAIndexTests(unittest.TestCase):
         feature4 = 'NADH010101'
 #1.)
         #testing index codes are in the AAI1
-        self.assertIn(feature1, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertIn(feature1, self.aaindex.record_codes(), 'Index {} \
             not found in list of available indices.'.format(feature1))
-        self.assertIn(feature2, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertIn(feature2, self.aaindex.record_codes(), 'Index {} \
             not found in list of available indices'.format(feature2))
-        self.assertIn(feature3, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertIn(feature3, self.aaindex.record_codes(), 'Index {} \
             not found in list of available indices'.format(feature3))
-        self.assertIn(feature4, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertIn(feature4, self.aaindex.record_codes(), 'Index {} \
             not found in list of available indices'.format(feature4))
 #2.)
         #testing bogus index codes
@@ -231,17 +256,19 @@ class AAIndexTests(unittest.TestCase):
         feature8 = 'ABC1234567'
 
         #testing errenous index codes are not in the AAI1
-        self.assertNotIn(feature5, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertNotIn(feature5, self.aaindex.record_codes(), 'Index {} \
             erroneously found in list of available indices.'.format(feature5))
-        self.assertNotIn(feature6, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertNotIn(feature6, self.aaindex.record_codes(), 'Index {} \
             erroneously found in list of available indices.'.format(feature6))
-        self.assertNotIn(feature7, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertNotIn(feature7, self.aaindex.record_codes(), 'Index {} \
             erroneously found in list of available indices.'.format(feature7))
-        self.assertNotIn(feature8, self.aaindex.get_record_codes(), 'Index {} \
+        self.assertNotIn(feature8, self.aaindex.record_codes(), 'Index {} \
             erroneously found in list of available indices.'.format(feature8))
 
     def test_category(self):
-        """ Test Case to check that the correct category value is returned for each index. """
+        """ 
+        Test Case to check that the correct category value is returned for each index. 
+        """
 
         feature1 = 'TANS770109'
         feature2 = 'RADA880107'
@@ -250,12 +277,12 @@ class AAIndexTests(unittest.TestCase):
         err_feature1 = "blahblahblah"
         err_feature2 = "12345"
         err_feature3 = 100.902
-#1.)
+
         cat1 = self.aaindex.get_category_from_record(feature1)
         cat2 = self.aaindex.get_category_from_record(feature2)
         cat3 = self.aaindex.get_category_from_record(feature3)
         cat4 = self.aaindex.get_category_from_record(feature4)
-
+#1.)
         self.assertEqual(cat1, 'sec_struct', 'Incorrect category returned, got {} \
             instead of {} '.format(cat1, 'sec_struct'))
         self.assertEqual(cat2, 'hydrophobic','Incorrect category returned, got {} \
@@ -275,43 +302,46 @@ class AAIndexTests(unittest.TestCase):
             cat7 = self.aaindex.get_category_from_record(err_feature3)
 
     def test_aaindex_names(self):
-        """ Test Case to check that the correct names/description is returned for each index. """
-
+        """ 
+        Test Case to check that the correct names/description is returned for each index. 
+        """
+        #get all record names
         names = self.aaindex.get_record_names()
 
         name1 = names[50]
         name2 = names[140]
         name3 = names[368]
         name4 = names[560]
-
+#1.)
         self.assertEqual(len(names), self.aaindex.get_num_records(), 'Total number \
             of names in the AAI database should equal the number of records')
-        self.assertEqual(name1, 'Frequency of the 3rd residue in turn (Chou-Fasman, 1978b)',
+        self.assertEqual(name1, 'alpha-NH chemical shifts (Bundi-Wuthrich, 1979)',
             'Incorrect index name returned, got {} instead of {}'.format(name1,
-                'Frequency of the 3rd residue in turn (Chou-Fasman, 1978b)'))
-        self.assertTrue(name2.startswith('Average relative probability'),
+                'alpha-NH chemical shifts (Bundi-Wuthrich, 1979)'))
+        self.assertTrue(name2.startswith('Number of full nonbonding orbitals'),
             'Incorrect index name returned, got {} instead of {}'.format(name2,
-                'Average relative probability'))
-        self.assertIn('chain reversal', name3, 'String {} not found in index name: \
-            {}'.format('chain reversal',name3))
-        self.assertTrue(name4.endswith('(Karkbara-Knisley, 2016)'), 'Index does \
-            not end with {}, got {}'.format('(Karkbara-Knisley, 2016)', name4))
+                'Number of full nonbonding orbitals'))
+        self.assertIn('Normalized frequency of alpha-helix', name3, 'String {} not found in index name: \
+            {}'.format('Normalized frequency of alpha-helix',name3))
+        self.assertTrue(name4.endswith('(Zhou-Zhou, 2004)'), 'Index does \
+            not end with {}, got {}'.format('(Zhou-Zhou, 2004)', name4))
 
     def test_aaindex_refs(self):
-        """ Test Case to check that the correct references are returned for each index. """
-
+        """ 
+        Test Case to check that the correct references are returned for each index. 
+        """
         feature1 = 'VELV850101'
         feature2 = 'QIAN880139'
         feature3 = 'NAKH900112'
         feature4 = 'LEVM760103'
         feature5 = "blah_blah"
         feature6 = 123
-#1.)
+
         ref1 = self.aaindex.get_ref_from_record(feature1)
         ref2 = self.aaindex.get_ref_from_record(feature2)
         ref3 = self.aaindex.get_ref_from_record(feature3)
         ref4 = self.aaindex.get_ref_from_record(feature4)
-
+#1.)
         self.assertTrue(ref1.startswith('Veljkovic'), 'Incorrect index reference, \
             got {} instead of {} '.format(ref1,'' ))
         self.assertIn("globular proteins using neural network models", ref2)
@@ -325,9 +355,11 @@ class AAIndexTests(unittest.TestCase):
             ref6 = self.aaindex.get_ref_from_record(feature6)
 
     def test_aaindex_encoding(self):
-        """ Testing amino acids one-hot encoding. """
-#1.)
+        """ 
+        Testing amino acids one-hot encoding. 
+        """
         encoding = self.aaindex.get_amino_acids_encoding()
+   #1.)
         self.assertEqual(encoding.shape, (20,20))
         self.assertIsInstance(encoding, np.ndarray)
 
